#ifndef __UEXEC_H
#define __UEXEC_H
#include "uexec.h"

#ifdef ICCAVR
#include <iom103.h>
#include <macros.h>
#endif

//
// task states
//
enum { 
       T_STOPPED,              // task is not running at all 
       T_CREATED,              // task ready to run for 1st time
       T_READY,                // task ready to run again
       T_WF_MSG,               // task waiting for msg
       T_WF_EOC_MSG,           // task waiting for EOC msg
       T_WF_SIGNAL,            // task waiting for a signal
       T_SLEEPING,             // task sleeping for a while
     };

                               // pointer or address as unsigned integer
typedef unsigned int addr_t;

//
// task control block (tcb)
//
typedef struct TaskControlBlock
{
	struct	TaskControlBlock* next; // pointer to next TCB 
	unsigned char  tid;			      /* task id */
   unsigned char  prior;            // priority 0 (highest) 255 (lowest) 
	unsigned char  state;		      /* task state */
	unsigned char  ticksTReady;      /* how many ticks does the task execute */
	addr_t   func;	                  /* current PC for task */
	unsigned int	stack_start;		/* stack low value */
	unsigned int	stack_end;			/* stack high value */
	unsigned int	sp;		         /* current stack pointer */
   unsigned char  signal;           // last signal number received
   unsigned char  sigTid;           // task id of task sending last signal 
	unsigned int	tmoTicks;	      // ticks left before timeout of task state
   unsigned char  numMsgOnQue;      // number of msgs waiting on que
   unsigned char* msgQueGetPtr;     // msg queue get pointer for reading msgs
   unsigned char* msgQuePutPtr;     // msg queue put pointer for saving msg
   unsigned char* msgQue1stPtr;     // msg queue start pointer 
   unsigned char* msgQueEndPtr;     // msg queue end pointer + 1 
   unsigned char  eocLineNum;       // eoc line in wf eoc msg state
} TaskControlBlock;

//
// task message queue data format
//
                                    // msg data area, circular queue
                                       // fromTid,byteLen,data bytes ...
                                       // fromTid,byteLen,data bytes
  //unsigned char msgQueData[NUM_TASKS][UEXC_MSG_QUE_SIZE]; moved to caller


//
// list of tasks attached to clock
//                                    
typedef struct AttachList_t
{
   unsigned int timer;                 // initial timer value in mills
   unsigned int ticks;                 // current timer value in ticks

} AttachList_t;
                                    
//
// task trace buffer format
//
typedef struct TaskTrace_t
{
   unsigned int  tick;                 // clock tick for this timer interrupt
   unsigned int  pc;                   // interrupted program counter value
   unsigned int  sp;                   // interrupted stack pointer value
   unsigned char tid;                  // interrupted task id (if any)
   unsigned char state;                // interrupted task's state (if any)
   unsigned char tmoTicks;             // interrupted task's time slice ticks
} TaskTrace_t;

//
// processor specific stuff
//


#define __SREG__ 0x3f
#define __SP_H__ 0x3e
#define __SP_L__ 0x3d
#define low(x) ((x)&0xff)
#define high(x) (((x)>>8)&0xff)
#define hlow(x) (((x)>>16)&0xff)
#define hhigh(x) (((x)>>24)&0xff)
#define __tmp_reg__  r0
#define __zero_reg__ r1
#define __upper_reg__ r24
#define __pc_rel__(x) ((x)*2)

asm("#define	TCCR1A	 0x2f");
asm("#define	TCCR1B	 0x2e");
asm("#define	TCNT1H	 0x2d");
asm("#define	TCNT1L	 0x2c");
asm("#define	OCR1AH	 0x2b");
asm("#define	OCR1AL	 0x2a");
asm("#define	OCR1BH	 0x29");
asm("#define	OCR1BL	 0x28");
asm("#define	ICR1H  	 0x27");
asm("#define	ICR1L	    0x26");
asm("#define   TCCR2     0x25");
asm("#define   TCNT2     0x24");
asm("#define   OCR2      0x23");
asm("#define	WDTCR	    0x21");
asm("#define	TIMSK	    0x37");
asm("#define	OCIE1A  	 4");
asm("#define   OCIE2     7");
asm("#define	TOIE1	    2");
asm("#define	OCIE0	    1");
asm("#define	TOIE0	    0");
asm("#define	TCCR0	    0x33");
asm("#define	TCNT0	    0x32");
asm("#define	OCR0	    0x31");
asm("#define	CTC0	    3");
asm("#define	CTC2	    3");
asm("#define _output_compare1a_ 0x18");

void UEXC_Schedule(void);
void UEXC_ResumeTask(void);
void UEXC_KillSelf(void);
void UEXC_StartTimer(void);
void UEXC_RestartTimer(void);
void UEXC_StartNewTask(void);
void UEXC_CheckTask(void);
#endif
