//
//
// UEXEC UART Interrupt handler test
//
//

#include <io.h>           
#include "uexec.h"
#include "uart.h"
#include "uartIntr.h"


unsigned char testIUartStack[500];
int testIUartTid;

void testIUart(void);
void displayHelp(void);

//
// main for test timerTask.c
//
int main(void)
{ 

   IUART_Init();      
    
   PPRINT("\r\ntestIUartIntr running\r\n");

   *LED = (U8)~0xff;
   
   testIUartTid = UEXC_CreateTask(
                     testIUart,                     // entry point
                     testIUartStack,                // stack ptr
                     sizeof(testIUartStack),        // stack size
                     0,                             // msgQ ptr
                     0,                             // msqQ size
                     (U16)UEXC_DEFAULT_TIME_SLICE,  // time slice mills
                     100);                          // task's priority

   
	UEXC_StartScheduler(); // won't return  

   return 0;
}


//
// testIUart task
//
extern void testIUart(void)
{
   U8 lineSent[]="\r\nWhat do you want? ";
   U8 lineRead[80];
   U8 c;
   U8 i;
   U8 j;
   
   displayHelp();
   
   while (1)
   {
      //
      // example using function calls
      //
      if ( IUART_Xmt_Line(lineSent, sizeof(lineSent), WAIT_FOREVER) == 0)
      {
         PPRINT("\r\ntestIUart IUART_Xmt_Line failed\r\n");
         continue;
      }

      if( IUART_Rcv_Line(lineRead, sizeof(lineRead), WAIT_FOREVER) == 0) 
      {
         PPRINT("\r\ntestIUart IUART_Rcv_Line failed\r\n");
         continue;
      }
      else
      {
         IPRINT("\r\ntestIUart lineRead: '"); 
         IPRINT(lineRead);
         IPRINT("'\r\n");

         if (strlen(lineRead) == 2 && lineRead[0] == '?') 
            displayHelp();
      }

      //
      // example using macros
      //
      IPRINT("\r\nType a line ending with ';'");
      IREAD(lineRead);
      IPRINT("\r\nYou Typed '"); IPRINT(lineRead); IPRINT("'\r\n");

      if (strlen(lineRead) == 2 && lineRead[0] == '?') 
         displayHelp();

      //
      // example of get and put char
      //
      IPRINT("\r\nType input ending with ';'");

      i = 0;                      
      while (1)
      {
         c = IGETCHAR();         
         lineRead[i++] = c;
         lineRead[i] = 0;
         if (c == ';') 
            break;
      }
      IPRINT("\r\nYou Typed '");

      for (j=0; j<i; ++j)
      {
         IPUTCHAR(&lineRead[j]);
      }
      IPRINT("'\r\n");   

   }
}

//
//  display help for line editing
//

   U8 l0[] = "\r\n";
   U8 l1[] = "\r\nbackspace  (delete char left of cursor)" ;
   U8 l2[] = "\r\ndelete     (delete char right of cursor)";
   U8 l3[] = "\r\ncontrol-U  (undo line edits)";
   U8 l4[] = "\r\ncontrol-C  (clear current line)";
   U8 l5[] = "\r\ncontrol-D  (display current line again)";
   U8 l6[] = "\r\ncontrol-P  (previous line recall, can use up arrow)" ;
   U8 l7[] = "\r\ncontrol-N  (next line recall, can use down arrow)" ;
   U8 l8[] = "\r\ncontrol-L  (left  one character, can use left arrow)";
   U8 l9[] = "\r\ncontrol-R  (right one character, can use right arrow)";
   U8 lA[] = "\r\ncontrol-B  (move cursor to beginning of line, can use home)";
   U8 lB[] = "\r\ncontrol-E  (move cursor to end of line, can use end)";
   U8 lC[] = "\r\n?          (print help)";

void displayHelp(void)
{
   IPRINT(l0);   
   IPRINT(l1);   
   IPRINT(l2);   
   IPRINT(l3);   
   IPRINT(l4);   
   IPRINT(l5);   
   IPRINT(l6);   
   IPRINT(l7);   
   IPRINT(l8);   
   IPRINT(l9);   
   IPRINT(lA);   
   IPRINT(lB);   
   IPRINT(lC);   
   IPRINT(l0);   
}
