//
//
// UEXEC Timer Task
//
//

#include "uexec.h"
#include "uart.h"
#include "timerTask.h"


                                       // timer table 
TimerEntry_t timerTbl[NUM_TIMER_TBL_ENTRIES];

                                       // timer task stack
unsigned char timerTaskStack[200];
                                       // timer task msg queue
unsigned char timerTaskMsgQue[100];

                                       // task id for TimerTask
unsigned char timerTaskTid;

void TimerTask(void);

//
// start a timer 
//
extern unsigned char StartTimer(
                            unsigned long seconds,      // number of seconds to time
                            unsigned int  mills,        // number of milliseconds to time
                            unsigned char fromTid,      // uexec task id making request
                            TimerReplyMsg_t* replyMsg,  // reply message data sent on tmo
                            TimerId_t* timerNum)        // timer number, for killing timer
{
   unsigned char idx, i;

   //
   // search for an empty timer table entry
   //
   for (idx=0; idx<NUM_TIMER_TBL_ENTRIES; ++idx) 
   {
     if (timerTbl[idx].tmoTicks == 0)
     {                                    
        //
        // make a timer table entry
        //
        timerTbl[idx].tmoTicks =           // set up initial tics
           (seconds*1000 + mills + TIMER_MILLS_PER_TICK -1)/ TIMER_MILLS_PER_TICK;
        
        if (timerTbl[idx].tmoTicks == 0)   // check for no tmo value
           return 1;

        timerTbl[idx].reqTid = fromTid;   // remember tid for reply msg
        
                                          // remember reply msg number
        timerTbl[idx].msgNum = replyMsg->msgNum;   
              
                                          // remember reply msg number
        for (i=0; i<MAX_TIMER_REPLY_DATA; ++i )     
        {
           timerTbl[idx].data[i] = replyMsg->data[i];
           //PPRINT("\r\nStartTimer timerTbl[idx].data[i] 0x");
           //PPRINTU8(timerTbl[idx].data[i]);
        }
                                          // return timer table index
        *timerNum = idx;                                

        return 1; // success
     }
   }
   return 0; // failed, all timer entries are busy
}

//
// cancel a timer
//
extern unsigned char KillTimer(TimerId_t timerNum)
{
   if (timerNum < NUM_TIMER_TBL_ENTRIES)
   {
     timerTbl[timerNum].tmoTicks = 0;
     return 1; // no more timer
   }

   return 0; // invalid timer entry number
}


//
// start up timer task
//
extern void InitTimerTask(void)
{
   unsigned int idx;
   
                                                    // clear all timers
   for (idx=0; idx<NUM_TIMER_TBL_ENTRIES; ++idx)
      timerTbl[idx].tmoTicks = 0; 
                                                    // start timer task
   timerTaskTid    = UEXC_CreateTask(              
                     TimerTask,                       // entry point
                     timerTaskStack,                  // stack ptr
                     sizeof(timerTaskStack),          // stack size
                     timerTaskMsgQue,                 // msgQ ptr  
                     sizeof(timerTaskMsgQue),         // msqQ size
                     (U16)UEXC_DEFAULT_TIME_SLICE,    // time slice mills
                     TIMER_TASK_PRIOR);               // task's priority (high)
}

//
// uexec task to perform timing for any task in system
//
extern void TimerTask(void)
{
   
   unsigned int ticks, idx;
   
   
   UEXC_Attach(0,TIMER_MILLS_PER_TICK); // attach to clock
   //
   // scan timer table entries
   //
   while (1)
   {
      //PPRINT("\r\nTimerTask calling UEXC_Sleep UEXC_SLEEP_ATTACHED");    
      UEXC_Sleep( UEXC_SLEEP_ATTACHED ,0);  // sleep until run off clock

      ticks = UEXC_GetClockTicks();         // get current clock ticks
      //PPRINT("\r\nTimerTask ticks 0x"); PPRINTU16(ticks);

      for (idx=0; idx<NUM_TIMER_TBL_ENTRIES; ++idx)
      {
         if (timerTbl[idx].tmoTicks > 0) 
         {
            //PPRINT("\r\nTimerTask timerTbl[");
            //PPRINTU8(idx); PPRINT("].tmoTicks 0x"); 
            //PPRINTU16(timerTbl[idx].tmoTicks);

            --timerTbl[idx].tmoTicks;

            if (timerTbl[idx].tmoTicks == 0) 
            {
               //PPRINT("\r\nTimerTask calling UEXC_SendMsg");

               if (UEXC_SendMsg(&timerTbl[idx].msgNum, 
                                sizeof(TimerReplyMsg_t), 
                                timerTaskTid, 
                                timerTbl[idx].reqTid))
               {
                  //PPRINT("\r\nnTimerTask sent msg of 0x"); 
                  //PPRINTU8(sizeof(TimerReplyMsg_t)); PPRINT(" bytes\r\n");
               }
               //else 
                  //PPRINT("\r\nTimerTask UEXC_SendMsg failed");
            }
         }

      }
   }
}
